document.getStreams = function () {
  return ['electrics', 'engineInfo'];
};

var width = 660;
var height = 660;
var initialized = false;
var rpm_redline_len = 0;
var dashSize = 7;

// elements

// speed
var speedText = document.getElementById('tacho2speed');
speedText.textContent = '';

// gear
var gearText = document.getElementById('tacho2gear');

// revcurve dashes
var revCurveDashes = document.getElementById('revcurve_dashes');
var revCurveDashesLen = revCurveDashes.getTotalLength();

// redline
var redLine = document.getElementById('rpm_redline');
var redLineLen = redLine.getTotalLength();

// rpmTextGuideLine -- a helping line to position the text
var rpmTextGuideLine = document.getElementById('rpmtextline');
var rpmTextGuideLineLen = rpmTextGuideLine.getTotalLength();
rpmTextGuideLine.style['display'] = 'none';

// the rpm text
var rpmTextSize = 50; // in px
var maxRpmTexts = 13;
// setup + hide all
for (var k = 1; k < maxRpmTexts; k++) {
  document.getElementById('rpmtext' + k).style['visibility'] = 'hidden';
  document.getElementById('rpmtext' + k).style['font-size'] = rpmTextSize + 'px';
}
var rpm_max = 8000;
//maxRpmTexts = 3; // this can artifically limit the number of rpm splits


// revneedle
var revNeedle = document.getElementById('revneedle');

// Icons
var icoHandBrakeOff = document.getElementById('ico_handbrake');
var icoHandBrakeOn = document.getElementById('ico_handbrake_on');
var icoABSOff = document.getElementById('ico_abs');
var icoABSOn = document.getElementById('ico_abs_on');
var icoIndicatorLeftOff = document.getElementById('ico_indicatorl');
var icoIndicatorLeftOn = document.getElementById('ico_indicatorl_on');
var icoIndicatorRightOff = document.getElementById('ico_indicatorr');
var icoIndicatorRightOn = document.getElementById('ico_indicatorr_on');

// Lights
var tachoGlow = document.getElementsByClassName("tacho_glow");
var icoLightsOn = document.getElementById('ico_lights_on');
var icoLightsHigh = document.getElementById('ico_lights_high');

var speedUnitText = document.getElementById('speedunit');

var testVar = 0; // for the test mode
var displayMode = 2;

function applyData(data) {
  speedText.textContent = data.speedtext;
  if (speedText.textContent == "-Infinity" || speedText.textContent == "Infinity") { speedText.textContent = "-" };
  gearText.textContent = data.geartext;

  icoHandBrakeOff.style['visibility'] = data.parkingBrake ? 'hidden' : 'visible';
  icoHandBrakeOn.style['visibility'] = data.parkingBrake ? 'visible' : 'hidden';

  icoABSOff.style['visibility'] = data.absWorking ? 'hidden' : 'visible';
  icoABSOn.style['visibility'] = data.absWorking ? 'visible' : 'hidden';

  icoIndicatorLeftOff.style['visibility'] = data.signalL ? 'hidden' : 'visible';
  icoIndicatorLeftOn.style['visibility'] = data.signalL ? 'visible' : 'hidden';

  icoIndicatorRightOff.style['visibility'] = data.signalR ? 'hidden' : 'visible';
  icoIndicatorRightOn.style['visibility'] = data.signalR ? 'visible' : 'hidden';

  if (typeof data.lowBeam !== 'undefined' && typeof data.highBeam !== 'undefined') {
    var lightsOff = true;
    var lb = (data.lowBeam > 0.9);
    var hb = (data.highBeam > 0.9);
    if (lb) { lightsOff = false; }
    if (hb) { lightsOff = false; }

    for (_i = 0; _i < tachoGlow.length; _i++) {
      tachoGlow[_i].style['display'] = lightsOff ? 'none' : 'inline';
    }
    icoLightsOn.style['visibility'] = lb ? 'visible' : 'hidden';
    icoLightsHigh.style['visibility'] = hb ? 'visible' : 'hidden';
  } else {
    for (_i = 0; _i < tachoGlow.length; _i++) {
      tachoGlow[_i].style['display'] = 'none';
    }
    icoLightsOn.style['visibility'] = 'hidden';
    icoLightsHigh.style['visibility'] = 'hidden';
  }

  var rpm_rotation = data.rpm * 270 - 180;
  if (rpm_rotation < -180) {
    rpm_rotation = -180;
  }
  if (rpm_rotation > 90) {
    rpm_rotation = 90;
  }
  revNeedle.setAttribute('transform', 'rotate(' + rpm_rotation + ',' + (width / 2) + ',' + (height / 2) + ')');
};

var data = {}; // local data to display, do not use by direct access, pass it to functions

var layersVisible = false;
document.setlayersVisible = function (v) {
  if (layersVisible != v) {
    var val = v ? 'inline' : 'none';
    document.getElementById('layer3').style['display'] = val;
    document.getElementById('layer4').style['display'] = val;
    document.getElementById('layer6').style['display'] = val;
    document.getElementById('layer7').style['display'] = val;
    document.getElementById('layer10').style['display'] = val;
    document.getElementById('layer11').style['display'] = val;
    layersVisible = v;
  }
}

document.reset = function () {
  document.setlayersVisible(false);
  initialized = 0;
  for (var k = 1; k < maxRpmTexts + 1; k++) {
    document.getElementById('rpmtext' + (k + 1)).style['visibility'] = 'hidden';
  }
};

document.update = function (streams) {
  var isStreamValid = streams.engineInfo && streams.engineInfo[1] !== undefined && streams.engineInfo[1] !== 0 && streams.electrics;

  //console.log(streams);

  if (!initialized) {

    document.reset();

    if (isStreamValid) {

      rpm_max = Math.round(streams.engineInfo[1] / 1000) * 1000 + 1000; // we round up to the nearest 1000

      // things that need to do once (and depending on the vehicle attributes)
      // var upshiftRPM_perc = (streams.engineInfo[1] / rpm_max);
      var upshiftRPM_perc = ((Math.floor((streams.engineInfo[1] - 250) / 500) * 500) / (rpm_max + 250));
      if (streams.engineInfo[1] == 0)
        upshiftRPM_perc = 1

      // always draw the full gauge no matter what
      var upshiftRPM2_perc = 1; //((streams.engineInfo[1] - streams.engineInfo[2]) / rpm_max);

      redLine.style['stroke-dasharray'] = (redLineLen * upshiftRPM2_perc) + ' ' + redLineLen;
      redLine.style['stroke-dashoffset'] = -upshiftRPM_perc * redLineLen;

      var dashCount = Math.round(rpm_max / 1000) * 2 - 1;
      if (dashCount > maxRpmTexts * 2 + 1) { dashCount = maxRpmTexts * 2 + 1; }

      dashSize = 6;
      dash2Size = 12;
      var dashes = (revCurveDashesLen - dashSize * (dashCount + 1)) / (dashCount + 1);
      var dashes2 = dashes - Math.abs(dash2Size - dashSize) * 1.0 / 2;
      revCurveDashes.style['stroke-dasharray'] = dash2Size + ' ' + dashes2 + ' ' + dashSize + ' ' + dashes2;
      revCurveDashes.style['stroke-dashoffset'] = (dash2Size - dashSize);

      var n = (dashCount + 1) / 2;
      // setup and move the rpm gauge text
      var rpmSlice = rpm_max / n / 1000;
      for (var k = 1; k < n; k++) {
        var t = Math.round(rpmSlice * k * 10) / 10;
        var pos = rpmTextGuideLine.getPointAtLength(rpmTextGuideLineLen * (k / n));
        var rp = document.getElementById('rpmtext' + (k + 1) + 'p');
        var cx = rp.getAttribute('x');
        var cy = rp.getAttribute('y');
        rp.setAttribute('transform', 'translate(' + (pos.x - cx) + ',' + (pos.y - cy + (rpmTextSize / 2)) + ')');
        document.getElementById('rpmtext' + (k + 1)).textContent = t;
        document.getElementById('rpmtext' + (k + 1)).style['visibility'] = 'visible';
      }

      //console.log(streams);
      initialized = true;
    }
  }

  if (!isStreamValid) {
    return false;
  }

  document.setlayersVisible(true);

  if (displayMode == 2) {
    // real data mode, no animation
    if (streams.electrics.wheelspeed) {
      data.speedtext = UnitSpeed(streams.electrics.wheelspeed);
      if (streams.electrics.wheelspeed > 9000) {
        speedUnitText.textContent = 'brrrr';
      }
    } else if (streams.electrics.airspeed) {
      data.speedtext = UnitSpeed(streams.electrics.airspeed);
    }

    (function () {
      if (streams.engineInfo[13] == "manual") {
        var gear = streams.engineInfo[5];
        var gearStr = gear.toString();

        if (gear == 0) gearStr = 'N';
        else if (gear == -1) gearStr = 'R';
        else if (-gear > 1) gearStr = 'R' + (-gear);

        data.geartext = gearStr;
      } else {
        data.geartext = ["P", "R", "N", "D", "2", "1"][Math.round(streams.electrics.gear_A * 5)];
      }
    })();

    data.parkingBrake = streams.electrics.parkingbrake;
    data.absWorking = streams.electrics.abs;
    data.signalL = streams.electrics.signal_L;
    data.signalR = streams.electrics.signal_R;
    data.lowBeam = streams.electrics.lowbeam;
    data.highBeam = streams.electrics.highbeam;
    data.rpm = (streams.electrics.rpmTacho || 0.0) / rpm_max;

  } else if (displayMode == 0) {
    // starting animation
    testVar = testVar + 0.04;
    if (testVar > 1) testVar = 1;
    data.speedtext = Math.round(testVar * 100);
    data.geartext = Math.round(testVar * 5);
    var boolTest = true;  //Math.round(testVar * 10) % 10;
    data.parkingBrake = boolTest;
    data.absWorking = boolTest;
    data.signalL = boolTest;
    data.signalR = boolTest;
    data.lowBeam = boolTest;
    data.highBeam = !boolTest;
    data.rpm = testVar;
    if (testVar >= 1) {
      testVar = 0;
      displayMode = 1;
    }

  } else if (displayMode == 1) {
    // starting animation -> real data transition

    if (streams.electrics.wheelspeed) {
      data.speedtext = UnitSpeed(streams.electrics.wheelspeed);
    } else {
      data.speedtext = '';
      speedUnitText.textContent = '';
    };

    (function () {
      var gear = streams.engineInfo[5];
      var gearStr = gear.toString();
      if (gear == 0) {
        gearStr = 'N';
      }
      else if (gear == -1) {
        gearStr = 'R';
      }
      data.geartext = gearStr;
    })()

    data.parkingBrake = streams.electrics.parkingbrake;
    data.absWorking = streams.electrics.abs;
    data.signalL = streams.electrics.signal_L;
    data.signalR = streams.electrics.signal_R;
    data.lowBeam = streams.electrics.lowbeam;
    data.highBeam = streams.electrics.highbeam;

    var rpmperc = streams.electrics.rpm / rpm_max;
    var rpmok = Math.abs(data.rpm - rpmperc) < 0.005;
    if (!rpmok) data.rpm += (rpmperc - data.rpm) * 0.2;

  }

  applyData(data);
  return true;
};

document.vehicleChanged = function () {
  // redo the elements 
  initialized = false;
};

var UiUnitscallback;

function UnitSpeed(val) {
  var convertedVal = UiUnitscallback(val, 'speed');
  speedUnitText.textContent = convertedVal.unit;
  return Math.round(convertedVal.val);
}

document.wireThroughUnitSystem = function (callback) {
  UiUnitscallback = callback;
};